
(* The type of tokens. *)

type token = 
  | RBRACKET
  | Open_equal
  | Open_dot
  | Open_colon
  | MINUS
  | LBRACKET
  | HAT
  | Equal_close
  | EOF
  | Dot_close
  | Colon_close
  | COLL_ELEM_SINGLE of (char)
  | COLL_ELEM_MULTI of (string)

(* This exception is raised by the monolithic API functions. *)

exception Error

(* The monolithic API. *)

val bracket_expression: (Lexing.lexbuf -> token) -> Lexing.lexbuf -> (CST.bracket_expression)

module MenhirInterpreter : sig
  
  (* The incremental API. *)
  
  include MenhirLib.IncrementalEngine.INCREMENTAL_ENGINE
    with type token = token
  
  (* The indexed type of terminal symbols. *)
  
  type _ terminal = 
    | T_error : unit terminal
    | T_RBRACKET : unit terminal
    | T_Open_equal : unit terminal
    | T_Open_dot : unit terminal
    | T_Open_colon : unit terminal
    | T_MINUS : unit terminal
    | T_LBRACKET : unit terminal
    | T_HAT : unit terminal
    | T_Equal_close : unit terminal
    | T_EOF : unit terminal
    | T_Dot_close : unit terminal
    | T_Colon_close : unit terminal
    | T_COLL_ELEM_SINGLE : (char) terminal
    | T_COLL_ELEM_MULTI : (string) terminal
  
  (* The indexed type of nonterminal symbols. *)
  
  type _ nonterminal = 
    | N_start_range : (CST.start_range) nonterminal
    | N_single_expression : (CST.single_expression) nonterminal
    | N_range_expression : (CST.range_expression) nonterminal
    | N_nonmatching_list : (CST.nonmatching_list) nonterminal
    | N_nonempty_list_COLL_ELEM_SINGLE_ : (char list) nonterminal
    | N_meta_char : (char) nonterminal
    | N_matching_list : (CST.matching_list) nonterminal
    | N_follow_list : (CST.follow_list) nonterminal
    | N_expression_term : (CST.expression_term) nonterminal
    | N_equivalence_class : (CST.equivalence_class) nonterminal
    | N_end_range : (CST.end_range) nonterminal
    | N_collating_symbol : (CST.collating_symbol) nonterminal
    | N_class_name : (string) nonterminal
    | N_character_class : (CST.character_class) nonterminal
    | N_bracket_list : (CST.bracket_list) nonterminal
    | N_bracket_expression : (CST.bracket_expression) nonterminal
  
  (* The inspection API. *)
  
  include MenhirLib.IncrementalEngine.INSPECTION
    with type 'a lr1state := 'a lr1state
    with type production := production
    with type 'a terminal := 'a terminal
    with type 'a nonterminal := 'a nonterminal
    with type 'a env := 'a env
  
end

(* The entry point(s) to the incremental API. *)

module Incremental : sig
  
  val bracket_expression: Lexing.position -> (CST.bracket_expression) MenhirInterpreter.checkpoint
  
end
